/**
 * boilerpipe
 *
 * Copyright (c) 2009 Christian Kohlschütter
 *
 * The author licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.l3s.boilerpipe;

/**
 * Exception for signaling failure in the processing pipeline.
 * 
 * @author Christian Kohlschütter
 */
public class BoilerpipeProcessingException extends Exception {
    private static final long serialVersionUID = 1L;

    public BoilerpipeProcessingException() {
        super();
    }

    public BoilerpipeProcessingException(String message, Throwable cause) {
        super(message, cause);
    }

    public BoilerpipeProcessingException(String message) {
        super(message);
    }

    public BoilerpipeProcessingException(Throwable cause) {
        super(cause);
    }
}
